@@ -8,7 +8,7 @@
    This file is part of Cachegrind, a Valgrind tool for cache
    profiling programs.
 
-   Copyright (C) 2002-2008 Nicholas Nethercote
+   Copyright (C) 2002-2009 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -1032,6 +1032,27 @@ IRSB* cg_instrument ( VgCallbackClosure* closure,
             break;
          }
 
+         case Ist_CAS: {
+            /* We treat it as a read and a write of the location.  I
+               think that is the same behaviour as it was before IRCAS
+               was introduced, since prior to that point, the Vex
+               front ends would translate a lock-prefixed instruction
+               into a (normal) read followed by a (normal) write. */
+            Int    dataSize;
+            IRCAS* cas = st->Ist.CAS.details;
+            tl_assert(cas->addr != NULL);
+            tl_assert(cas->dataLo != NULL);
+            dataSize = sizeofIRType(typeOfIRExpr(tyenv, cas->dataLo));
+            if (cas->dataHi != NULL)
+               dataSize *= 2; /* since it's a doubleword-CAS */
+            /* I don't think this can ever happen, but play safe. */
+            if (dataSize > MIN_LINE_SIZE)
+               dataSize = MIN_LINE_SIZE;
+            addEvent_Dr( &cgs, curr_inode, dataSize, cas->addr );
+            addEvent_Dw( &cgs, curr_inode, dataSize, cas->addr );
+            break;
+         }
+
          case Ist_Exit: {
             /* Stuff to widen the guard expression to a host word, so
                we can pass it to the branch predictor simulation
@@ -1161,16 +1182,13 @@ void check_cache(cache_t* cache, Char *name)
    /* Simulator requires line size and set count to be powers of two */
    if (( cache->size % (cache->line_size * cache->assoc) != 0) ||
        (-1 == VG_(log2)(cache->size/cache->line_size/cache->assoc))) {
-      VG_(message)(Vg_UserMsg,
-         "error: %s set count not a power of two; aborting.",
-         name);
+      VG_(umsg)("error: %s set count not a power of two; aborting.\n", name);
       VG_(exit)(1);
    }
 
    if (-1 == VG_(log2)(cache->line_size)) {
-      VG_(message)(Vg_UserMsg,
-         "error: %s line size of %dB not a power of two; aborting.",
-         name, cache->line_size);
+      VG_(umsg)("error: %s line size of %dB not a power of two; aborting.\n",
+                name, cache->line_size);
       VG_(exit)(1);
    }
 
@@ -1178,24 +1196,22 @@ void check_cache(cache_t* cache, Char *name)
    // straddle three cache lines, which breaks a simulation assertion and is
    // stupid anyway.
    if (cache->line_size < MIN_LINE_SIZE) {
-      VG_(message)(Vg_UserMsg,
-         "error: %s line size of %dB too small; aborting.", 
-         name, cache->line_size);
+      VG_(umsg)("error: %s line size of %dB too small; aborting.\n", 
+                name, cache->line_size);
       VG_(exit)(1);
    }
 
    /* Then check cache size > line size (causes seg faults if not). */
    if (cache->size <= cache->line_size) {
-      VG_(message)(Vg_UserMsg,
-         "error: %s cache size of %dB <= line size of %dB; aborting.",
-         name, cache->size, cache->line_size);
+      VG_(umsg)("error: %s cache size of %dB <= line size of %dB; aborting.\n",
+                name, cache->size, cache->line_size);
       VG_(exit)(1);
    }
 
    /* Then check assoc <= (size / line size) (seg faults otherwise). */
    if (cache->assoc > (cache->size / cache->line_size)) {
-      VG_(message)(Vg_UserMsg,
-         "warning: %s associativity > (size / line size); aborting.", name);
+      VG_(umsg)("warning: %s associativity > (size / line size); aborting.\n",
+                name);
       VG_(exit)(1);
    }
 }
@@ -1226,14 +1242,14 @@ void configure_caches(cache_t* I1c, cache_t* D1c, cache_t* L2c)
    check_cache(D1c, "D1");
    check_cache(L2c, "L2");
 
-   if (VG_(clo_verbosity) > 1) {
-      VG_(message)(Vg_UserMsg, "Cache configuration used:");
-      VG_(message)(Vg_UserMsg, "  I1: %dB, %d-way, %dB lines",
-                               I1c->size, I1c->assoc, I1c->line_size);
-      VG_(message)(Vg_UserMsg, "  D1: %dB, %d-way, %dB lines",
-                               D1c->size, D1c->assoc, D1c->line_size);
-      VG_(message)(Vg_UserMsg, "  L2: %dB, %d-way, %dB lines",
-                               L2c->size, L2c->assoc, L2c->line_size);
+   if (VG_(clo_verbosity) >= 2) {
+      VG_(umsg)("Cache configuration used:\n");
+      VG_(umsg)("  I1: %dB, %d-way, %dB lines\n",
+                I1c->size, I1c->assoc, I1c->line_size);
+      VG_(umsg)("  D1: %dB, %d-way, %dB lines\n",
+                D1c->size, D1c->assoc, D1c->line_size);
+      VG_(umsg)("  L2: %dB, %d-way, %dB lines\n",
+                L2c->size, L2c->assoc, L2c->line_size);
    }
 #undef CMD_LINE_DEFINED
 }
@@ -1267,18 +1283,16 @@ static void fprint_CC_table_and_calc_totals(void)
 
    sres = VG_(open)(cachegrind_out_file, VKI_O_CREAT|VKI_O_TRUNC|VKI_O_WRONLY,
                                          VKI_S_IRUSR|VKI_S_IWUSR);
-   if (sres.isError) {
+   if (sr_isError(sres)) {
       // If the file can't be opened for whatever reason (conflict
       // between multiple cachegrinded processes?), give up now.
-      VG_(message)(Vg_UserMsg,
-         "error: can't open cache simulation output file '%s'",
-         cachegrind_out_file );
-      VG_(message)(Vg_UserMsg,
-         "       ... so simulation results will be missing.");
+      VG_(umsg)("error: can't open cache simulation output file '%s'\n",
+                cachegrind_out_file );
+      VG_(umsg)("       ... so simulation results will be missing.\n");
       VG_(free)(cachegrind_out_file);
       return;
    } else {
-      fd = sres.res;
+      fd = sr_Res(sres);
       VG_(free)(cachegrind_out_file);
    }
 
@@ -1348,7 +1362,6 @@ static void fprint_CC_table_and_calc_totals(void)
          VG_(sprintf)(buf, "fn=%s\n", currFn);
          VG_(write)(fd, (void*)buf, VG_(strlen)(buf));
          distinct_fns++;
-         just_hit_a_new_file = False;
       }
 
       // Print the LineCC
@@ -1461,8 +1474,7 @@ static void cg_fini(Int exitcode)
    BranchCC B_total;
    ULong L2_total_m, L2_total_mr, L2_total_mw,
          L2_total, L2_total_r, L2_total_w;
-   Int l1, l2, l3, l4;
-   Int p;
+   Int l1, l2, l3;
 
    /* Running with both cache and branch simulation disabled is not
       allowed (checked during command line option processing). */
@@ -1473,34 +1485,34 @@ static void cg_fini(Int exitcode)
    if (VG_(clo_verbosity) == 0) 
       return;
 
+   // Nb: this isn't called "MAX" because that overshadows a global on Darwin.
+   #define CG_MAX(a, b)  ((a) >= (b) ? (a) : (b))
+
    /* I cache results.  Use the I_refs value to determine the first column
     * width. */
    l1 = ULong_width(Ir_total.a);
-   l2 = ULong_width(Dr_total.a);
-   l3 = ULong_width(Dw_total.a);
-   l4 = ULong_width(Bc_total.b + Bi_total.b);
+   l2 = ULong_width(CG_MAX(Dr_total.a, Bc_total.b));
+   l3 = ULong_width(CG_MAX(Dw_total.a, Bi_total.b));
 
    /* Make format string, getting width right for numbers */
-   VG_(sprintf)(fmt, "%%s %%,%dllu", l1);
+   VG_(sprintf)(fmt, "%%s %%,%dllu\n", l1);
 
    /* Always print this */
-   VG_(message)(Vg_UserMsg, fmt, "I   refs:     ", Ir_total.a);
+   VG_(umsg)(fmt, "I   refs:     ", Ir_total.a);
 
    /* If cache profiling is enabled, show D access numbers and all
       miss numbers */
    if (clo_cache_sim) {
-      VG_(message)(Vg_UserMsg, fmt, "I1  misses:   ", Ir_total.m1);
-      VG_(message)(Vg_UserMsg, fmt, "L2i misses:   ", Ir_total.m2);
-
-      p = 100;
+      VG_(umsg)(fmt, "I1  misses:   ", Ir_total.m1);
+      VG_(umsg)(fmt, "L2i misses:   ", Ir_total.m2);
 
       if (0 == Ir_total.a) Ir_total.a = 1;
       VG_(percentify)(Ir_total.m1, Ir_total.a, 2, l1+1, buf1);
-      VG_(message)(Vg_UserMsg, "I1  miss rate: %s", buf1);
+      VG_(umsg)("I1  miss rate: %s\n", buf1);
 
       VG_(percentify)(Ir_total.m2, Ir_total.a, 2, l1+1, buf1);
-      VG_(message)(Vg_UserMsg, "L2i miss rate: %s", buf1);
-      VG_(message)(Vg_UserMsg, "");
+      VG_(umsg)("L2i miss rate: %s\n", buf1);
+      VG_(umsg)("\n");
 
       /* D cache results.  Use the D_refs.rd and D_refs.wr values to
        * determine the width of columns 2 & 3. */
@@ -1509,16 +1521,15 @@ static void cg_fini(Int exitcode)
       D_total.m2 = Dr_total.m2 + Dw_total.m2;
 
       /* Make format string, getting width right for numbers */
-      VG_(sprintf)(fmt, "%%s %%,%dllu  (%%,%dllu rd   + %%,%dllu wr)", l1, l2, l3);
+      VG_(sprintf)(fmt, "%%s %%,%dllu  (%%,%dllu rd   + %%,%dllu wr)\n",
+                        l1, l2, l3);
 
-      VG_(message)(Vg_UserMsg, fmt, "D   refs:     ", 
-                               D_total.a, Dr_total.a, Dw_total.a);
-      VG_(message)(Vg_UserMsg, fmt, "D1  misses:   ",
-                               D_total.m1, Dr_total.m1, Dw_total.m1);
-      VG_(message)(Vg_UserMsg, fmt, "L2d misses:   ",
-                               D_total.m2, Dr_total.m2, Dw_total.m2);
-
-      p = 10;
+      VG_(umsg)(fmt, "D   refs:     ", 
+                     D_total.a, Dr_total.a, Dw_total.a);
+      VG_(umsg)(fmt, "D1  misses:   ",
+                     D_total.m1, Dr_total.m1, Dw_total.m1);
+      VG_(umsg)(fmt, "L2d misses:   ",
+                     D_total.m2, Dr_total.m2, Dw_total.m2);
 
       if (0 == D_total.a)  D_total.a = 1;
       if (0 == Dr_total.a) Dr_total.a = 1;
@@ -1526,89 +1537,90 @@ static void cg_fini(Int exitcode)
       VG_(percentify)( D_total.m1,  D_total.a, 1, l1+1, buf1);
       VG_(percentify)(Dr_total.m1, Dr_total.a, 1, l2+1, buf2);
       VG_(percentify)(Dw_total.m1, Dw_total.a, 1, l3+1, buf3);
-      VG_(message)(Vg_UserMsg, "D1  miss rate: %s (%s     + %s  )", buf1, buf2,buf3);
+      VG_(umsg)("D1  miss rate: %s (%s     + %s  )\n", buf1, buf2,buf3);
 
       VG_(percentify)( D_total.m2,  D_total.a, 1, l1+1, buf1);
       VG_(percentify)(Dr_total.m2, Dr_total.a, 1, l2+1, buf2);
       VG_(percentify)(Dw_total.m2, Dw_total.a, 1, l3+1, buf3);
-      VG_(message)(Vg_UserMsg, "L2d miss rate: %s (%s     + %s  )", buf1, buf2,buf3);
-      VG_(message)(Vg_UserMsg, "");
+      VG_(umsg)("L2d miss rate: %s (%s     + %s  )\n", buf1, buf2,buf3);
+      VG_(umsg)("\n");
 
       /* L2 overall results */
 
       L2_total   = Dr_total.m1 + Dw_total.m1 + Ir_total.m1;
       L2_total_r = Dr_total.m1 + Ir_total.m1;
       L2_total_w = Dw_total.m1;
-      VG_(message)(Vg_UserMsg, fmt, "L2 refs:      ",
-                               L2_total, L2_total_r, L2_total_w);
+      VG_(umsg)(fmt, "L2 refs:      ",
+                     L2_total, L2_total_r, L2_total_w);
 
       L2_total_m  = Dr_total.m2 + Dw_total.m2 + Ir_total.m2;
       L2_total_mr = Dr_total.m2 + Ir_total.m2;
       L2_total_mw = Dw_total.m2;
-      VG_(message)(Vg_UserMsg, fmt, "L2 misses:    ",
-                               L2_total_m, L2_total_mr, L2_total_mw);
+      VG_(umsg)(fmt, "L2 misses:    ",
+                     L2_total_m, L2_total_mr, L2_total_mw);
 
       VG_(percentify)(L2_total_m,  (Ir_total.a + D_total.a),  1, l1+1, buf1);
       VG_(percentify)(L2_total_mr, (Ir_total.a + Dr_total.a), 1, l2+1, buf2);
       VG_(percentify)(L2_total_mw, Dw_total.a,                1, l3+1, buf3);
-      VG_(message)(Vg_UserMsg, "L2 miss rate:  %s (%s     + %s  )", buf1, buf2,buf3);
+      VG_(umsg)("L2 miss rate:  %s (%s     + %s  )\n", buf1, buf2,buf3);
    }
 
    /* If branch profiling is enabled, show branch overall results. */
    if (clo_branch_sim) {
       /* Make format string, getting width right for numbers */
-      VG_(sprintf)(fmt, "%%s %%,%dllu  (%%,%dllu cond + %%,%dllu ind)", l1, l2, l3);
+      VG_(sprintf)(fmt, "%%s %%,%dllu  (%%,%dllu cond + %%,%dllu ind)\n",
+                        l1, l2, l3);
 
       if (0 == Bc_total.b)  Bc_total.b = 1;
       if (0 == Bi_total.b)  Bi_total.b = 1;
       B_total.b  = Bc_total.b  + Bi_total.b;
       B_total.mp = Bc_total.mp + Bi_total.mp;
 
-      VG_(message)(Vg_UserMsg, "");
-      VG_(message)(Vg_UserMsg, fmt, "Branches:     ",
-                               B_total.b, Bc_total.b, Bi_total.b);
+      VG_(umsg)("\n");
+      VG_(umsg)(fmt, "Branches:     ",
+                     B_total.b, Bc_total.b, Bi_total.b);
 
-      VG_(message)(Vg_UserMsg, fmt, "Mispredicts:  ",
-                               B_total.mp, Bc_total.mp, Bi_total.mp);
+      VG_(umsg)(fmt, "Mispredicts:  ",
+                     B_total.mp, Bc_total.mp, Bi_total.mp);
 
       VG_(percentify)(B_total.mp,  B_total.b,  1, l1+1, buf1);
       VG_(percentify)(Bc_total.mp, Bc_total.b, 1, l2+1, buf2);
       VG_(percentify)(Bi_total.mp, Bi_total.b, 1, l3+1, buf3);
 
-      VG_(message)(Vg_UserMsg, "Mispred rate:  %s (%s     + %s   )", buf1, buf2,buf3);
+      VG_(umsg)("Mispred rate:  %s (%s     + %s   )\n", buf1, buf2,buf3);
    }
 
    // Various stats
-   if (VG_(clo_verbosity) > 1) {
+   if (VG_(clo_stats)) {
       Int debug_lookups = full_debugs      + fn_debugs +
                           file_line_debugs + no_debugs;
 
-      VG_(message)(Vg_DebugMsg, "");
-      VG_(message)(Vg_DebugMsg, "cachegrind: distinct files: %d", distinct_files);
-      VG_(message)(Vg_DebugMsg, "cachegrind: distinct fns:   %d", distinct_fns);
-      VG_(message)(Vg_DebugMsg, "cachegrind: distinct lines: %d", distinct_lines);
-      VG_(message)(Vg_DebugMsg, "cachegrind: distinct instrs:%d", distinct_instrs);
-      VG_(message)(Vg_DebugMsg, "cachegrind: debug lookups      : %d", debug_lookups);
+      VG_(dmsg)("\n");
+      VG_(dmsg)("cachegrind: distinct files: %d\n", distinct_files);
+      VG_(dmsg)("cachegrind: distinct fns:   %d\n", distinct_fns);
+      VG_(dmsg)("cachegrind: distinct lines: %d\n", distinct_lines);
+      VG_(dmsg)("cachegrind: distinct instrs:%d\n", distinct_instrs);
+      VG_(dmsg)("cachegrind: debug lookups      : %d\n", debug_lookups);
       
       VG_(percentify)(full_debugs,      debug_lookups, 1, 6, buf1);
       VG_(percentify)(file_line_debugs, debug_lookups, 1, 6, buf2);
       VG_(percentify)(fn_debugs,        debug_lookups, 1, 6, buf3);
       VG_(percentify)(no_debugs,        debug_lookups, 1, 6, buf4);
-      VG_(message)(Vg_DebugMsg, "cachegrind: with full      info:%s (%d)", 
-                   buf1, full_debugs);
-      VG_(message)(Vg_DebugMsg, "cachegrind: with file/line info:%s (%d)", 
-                   buf2, file_line_debugs);
-      VG_(message)(Vg_DebugMsg, "cachegrind: with fn name   info:%s (%d)", 
-                   buf3, fn_debugs);
-      VG_(message)(Vg_DebugMsg, "cachegrind: with zero      info:%s (%d)", 
-                   buf4, no_debugs);
+      VG_(dmsg)("cachegrind: with full      info:%s (%d)\n", 
+                buf1, full_debugs);
+      VG_(dmsg)("cachegrind: with file/line info:%s (%d)\n", 
+                buf2, file_line_debugs);
+      VG_(dmsg)("cachegrind: with fn name   info:%s (%d)\n", 
+                buf3, fn_debugs);
+      VG_(dmsg)("cachegrind: with zero      info:%s (%d)\n", 
+                buf4, no_debugs);
 
-      VG_(message)(Vg_DebugMsg, "cachegrind: string table size: %lu",
-                   VG_(OSetGen_Size)(stringTable));
-      VG_(message)(Vg_DebugMsg, "cachegrind: CC table size: %lu",
-                   VG_(OSetGen_Size)(CC_table));
-      VG_(message)(Vg_DebugMsg, "cachegrind: InstrInfo table size: %lu",
-                   VG_(OSetGen_Size)(instrInfoTable));
+      VG_(dmsg)("cachegrind: string table size: %lu\n",
+                VG_(OSetGen_Size)(stringTable));
+      VG_(dmsg)("cachegrind: CC table size: %lu\n",
+                VG_(OSetGen_Size)(CC_table));
+      VG_(dmsg)("cachegrind: InstrInfo table size: %lu\n",
+                VG_(OSetGen_Size)(instrInfoTable));
    }
 }
 
@@ -1645,50 +1657,47 @@ void cg_discard_superblock_info ( Addr64 orig_addr64, VexGuestExtents vge )
 
 static void parse_cache_opt ( cache_t* cache, Char* opt )
 {
-   Int i = 0, i2, i3;
+   Long i1, i2, i3;
+   Char* endptr;
 
-   // Option argument looks like "65536,2,64".
-   // Find commas, replace with NULs to make three independent 
-   // strings, then extract numbers, put NULs back.  Yuck.
-   while (VG_(isdigit)(opt[i])) i++;
-   if (',' == opt[i]) {
-      opt[i++] = '\0';
-      i2 = i;
-   } else goto bad;
-   while (VG_(isdigit)(opt[i])) i++;
-   if (',' == opt[i]) {
-      opt[i++] = '\0';
-      i3 = i;
-   } else goto bad;
-   while (VG_(isdigit)(opt[i])) i++;
-   if ('\0' != opt[i]) goto bad;
+   // Option argument looks like "65536,2,64".  Extract them.
+   i1 = VG_(strtoll10)(opt,      &endptr); if (*endptr != ',')  goto bad;
+   i2 = VG_(strtoll10)(endptr+1, &endptr); if (*endptr != ',')  goto bad;
+   i3 = VG_(strtoll10)(endptr+1, &endptr); if (*endptr != '\0') goto bad;
 
-   cache->size      = (Int)VG_(atoll)(opt);
-   cache->assoc     = (Int)VG_(atoll)(opt + i2);
-   cache->line_size = (Int)VG_(atoll)(opt + i3);
+   // Check for overflow.
+   cache->size      = (Int)i1;
+   cache->assoc     = (Int)i2;
+   cache->line_size = (Int)i3;
+   if (cache->size      != i1) goto overflow;
+   if (cache->assoc     != i2) goto overflow;
+   if (cache->line_size != i3) goto overflow;
 
-   opt[i2-1] = ',';
-   opt[i3-1] = ',';
    return;
 
+  overflow:
+   VG_(umsg)("one of the cache parameters was too large and overflowed\n");
   bad:
+   // XXX: this omits the "--I1/D1/L2=" part from the message, but that's
+   // not a big deal.
    VG_(err_bad_option)(opt);
 }
 
 static Bool cg_process_cmd_line_option(Char* arg)
 {
+   Char* tmp_str;
+
    // 5 is length of "--I1="
-   if      (VG_CLO_STREQN(5, arg, "--I1="))
-      parse_cache_opt(&clo_I1_cache, &arg[5]);
-   else if (VG_CLO_STREQN(5, arg, "--D1="))
-      parse_cache_opt(&clo_D1_cache, &arg[5]);
-   else if (VG_CLO_STREQN(5, arg, "--L2="))
-      parse_cache_opt(&clo_L2_cache, &arg[5]);
-   else if (VG_CLO_STREQN(22, arg, "--cachegrind-out-file=")) {
-      clo_cachegrind_out_file = &arg[22];
-   }
-   else VG_BOOL_CLO(arg, "--cache-sim",  clo_cache_sim)
-   else VG_BOOL_CLO(arg, "--branch-sim", clo_branch_sim)
+   if      VG_STR_CLO(arg, "--I1", tmp_str)
+      parse_cache_opt(&clo_I1_cache, tmp_str);
+   else if VG_STR_CLO(arg, "--D1", tmp_str)
+      parse_cache_opt(&clo_D1_cache, tmp_str);
+   else if VG_STR_CLO(arg, "--L2", tmp_str)
+      parse_cache_opt(&clo_L2_cache, tmp_str);
+
+   else if VG_STR_CLO( arg, "--cachegrind-out-file", clo_cachegrind_out_file) {}
+   else if VG_BOOL_CLO(arg, "--cache-sim",  clo_cache_sim)  {}
+   else if VG_BOOL_CLO(arg, "--branch-sim", clo_branch_sim) {}
    else
       return False;
 
@@ -1726,7 +1735,7 @@ static void cg_pre_clo_init(void)
    VG_(details_version)         (NULL);
    VG_(details_description)     ("a cache and branch-prediction profiler");
    VG_(details_copyright_author)(
-      "Copyright (C) 2002-2008, and GNU GPL'd, by Nicholas Nethercote et al.");
+      "Copyright (C) 2002-2009, and GNU GPL'd, by Nicholas Nethercote et al.");
    VG_(details_bug_reports_to)  (VG_BUGS_TO);
    VG_(details_avg_translation_sizeB) ( 500 );
 
@@ -1746,10 +1755,9 @@ static void cg_post_clo_init(void)
 
    /* Can't disable both cache and branch profiling */
    if ((!clo_cache_sim) && (!clo_branch_sim)) {
-      VG_(message)(Vg_DebugMsg,
-                   "ERROR: --cache-sim=no --branch-sim=no is not allowed.");
-      VG_(message)(Vg_DebugMsg,
-                   "You must select cache profiling, or branch profiling, or both.");
+      VG_(umsg)("ERROR: --cache-sim=no --branch-sim=no is not allowed.\n");
+      VG_(umsg)("You must select cache profiling, "
+                "or branch profiling, or both.\n");
       VG_(exit)(2);
    }
 
