@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2006-2008 OpenWorks LLP
+   Copyright (C) 2006-2009 OpenWorks LLP
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -33,6 +33,8 @@
    without prior written permission.
 */
 
+#if defined(VGO_aix5)
+
 /* This file reads XCOFF symbol tables and debug info.
    Known limitations:
 
@@ -58,6 +60,7 @@
 #include "pub_core_xarray.h"
 #include "priv_misc.h"
 #include "priv_tytypes.h"
+#include "pub_tool_debuginfo.h"
 #include "priv_d3basics.h"
 #include "priv_storage.h"
 #include "priv_readxcoff.h"        /* self */
@@ -1535,7 +1538,7 @@ HChar* read_symbol_table (
                         "fixme-Name-printing(5)" /*s->name*/ );
 
 	 if (guessed_toc)
-            VG_(message)(Vg_DebugMsg, "WARNING: assuming toc 0x%lx for %s", 
+            VG_(message)(Vg_DebugMsg, "WARNING: assuming toc 0x%lx for %s\n", 
                                       s->r2value, dis.name);
       }
    }
@@ -2421,10 +2424,10 @@ Bool ML_(read_xcoff_debug_info) ( struct _DebugInfo* di,
 
    if (VG_(clo_verbosity) > 1 || VG_(clo_trace_redir)) {
       if (di->memname) {
-         VG_(message)(Vg_DebugMsg, "Reading syms from %s(%s) (%#lx)",
+         VG_(message)(Vg_DebugMsg, "Reading syms from %s(%s) (%#lx)\n",
                       di->filename, di->memname, di->text_avma);
       } else {
-         VG_(message)(Vg_DebugMsg, "Reading syms from %s (%#lx)",
+         VG_(message)(Vg_DebugMsg, "Reading syms from %s (%#lx)\n",
                       di->filename, di->text_avma);
       }
    }
@@ -2480,6 +2483,8 @@ Bool ML_(read_xcoff_debug_info) ( struct _DebugInfo* di,
    return ok;
 }
 
+#endif // defined(VGO_aix5)
+
 /*--------------------------------------------------------------------*/
 /*--- end                                                          ---*/
 /*--------------------------------------------------------------------*/
