@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2005-2008 Nicholas Nethercote
+   Copyright (C) 2005-2009 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -138,7 +138,7 @@ static void resize ( VgHashTable table )
 }
 
 /* Puts a new, heap allocated VgHashNode, into the VgHashTable.  Prepends
-   the node to the appropriate chain. */
+   the node to the appropriate chain.  No duplicate key detection is done. */
 void VG_(HT_add_node) ( VgHashTable table, void* vnode )
 {
    VgHashNode* node     = (VgHashNode*)vnode;
@@ -190,10 +190,9 @@ void* VG_(HT_remove) ( VgHashTable table, UWord key )
    return NULL;
 }
 
-/* Allocates a suitably-sized array, copies all the hashtable elements
-   into it, then returns both the array and the size of it.  This is
-   used by the memory-leak detector.  The array must be freed with
-   VG_(free).
+/* Allocates a suitably-sized array, copies pointers to all the hashtable
+   elements into it, then returns both the array and the size of it.  The
+   array must be freed with VG_(free).
 */
 VgHashNode** VG_(HT_to_array) ( VgHashTable table, /*OUT*/ UInt* n_elems )
 {
@@ -201,12 +200,7 @@ VgHashNode** VG_(HT_to_array) ( VgHashTable table, /*OUT*/ UInt* n_elems )
    VgHashNode** arr;
    VgHashNode*  node;
 
-   *n_elems = 0;
-   for (i = 0; i < table->n_chains; i++) {
-      for (node = table->chains[i]; node != NULL; node = node->next) {
-         (*n_elems)++;
-      }
-   }
+   *n_elems = table->n_elements;
    if (*n_elems == 0)
       return NULL;
 
