@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2008 Julian Seward 
+   Copyright (C) 2000-2009 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -42,6 +42,17 @@
 #include "pub_core_xarray.h"
 #include "pub_core_clientstate.h"
 
+#if defined(VGO_darwin)
+/* --- !!! --- EXTERNAL HEADERS start --- !!! --- */
+#include <mach/mach.h>   /* mach_thread_self */
+/* --- !!! --- EXTERNAL HEADERS end --- !!! --- */
+#endif
+
+/* IMPORTANT: on Darwin it is essential to use the _nocancel versions
+   of syscalls rather than the vanilla version, if a _nocancel version
+   is available.  See docs/internals/Darwin-notes.txt for the reason
+   why. */
+
 /* ---------------------------------------------------------------------
    Command line and environment stuff
    ------------------------------------------------------------------ */
@@ -128,6 +139,7 @@ Char **VG_(env_setenv) ( Char ***envp, const Char* varname, const Char *val )
    return oldenv;
 }
 
+
 /* Walk through a colon-separated environment variable, and remove the
    entries which match remove_pattern.  It slides everything down over
    the removed entries, and pads the remaining space with '\0'.  It
@@ -193,17 +205,32 @@ static void mash_colon_env(Char *varp, const Char *remove_pattern)
 // when starting child processes, so they don't see that added stuff.
 void VG_(env_remove_valgrind_env_stuff)(Char** envp)
 {
+
+#if defined(VGO_darwin)
+
+   // Environment cleanup is also handled during parent launch 
+   // in vg_preloaded.c:vg_cleanup_env().
+
+#endif
+
    Int i;
    Char* ld_preload_str = NULL;
    Char* ld_library_path_str = NULL;
+   Char* dyld_insert_libraries_str = NULL;
    Char* buf;
 
    // Find LD_* variables
+   // DDD: should probably conditionally compiled some of this:
+   // - LD_LIBRARY_PATH is universal?
+   // - LD_PRELOAD is on Linux, not on Darwin, not sure about AIX
+   // - DYLD_INSERT_LIBRARIES and DYLD_SHARED_REGION are Darwin-only
    for (i = 0; envp[i] != NULL; i++) {
       if (VG_(strncmp)(envp[i], "LD_PRELOAD=", 11) == 0)
          ld_preload_str = &envp[i][11];
       if (VG_(strncmp)(envp[i], "LD_LIBRARY_PATH=", 16) == 0)
          ld_library_path_str = &envp[i][16];
+      if (VG_(strncmp)(envp[i], "DYLD_INSERT_LIBRARIES=", 22) == 0)
+         dyld_insert_libraries_str = &envp[i][22];
    }
 
    buf = VG_(arena_malloc)(VG_AR_CORE, "libcproc.erves.1",
@@ -212,12 +239,16 @@ void VG_(env_remove_valgrind_env_stuff)(Char** envp)
    // Remove Valgrind-specific entries from LD_*.
    VG_(sprintf)(buf, "%s*/vgpreload_*.so", VG_(libdir));
    mash_colon_env(ld_preload_str, buf);
+   mash_colon_env(dyld_insert_libraries_str, buf);
    VG_(sprintf)(buf, "%s*", VG_(libdir));
    mash_colon_env(ld_library_path_str, buf);
 
    // Remove VALGRIND_LAUNCHER variable.
    VG_(env_unsetenv)(envp, VALGRIND_LAUNCHER);
 
+   // Remove DYLD_SHARED_REGION variable.
+   VG_(env_unsetenv)(envp, "DYLD_SHARED_REGION");
+
    // XXX if variable becomes empty, remove it completely?
 
    VG_(arena_free)(VG_AR_CORE, buf);
@@ -230,16 +261,21 @@ void VG_(env_remove_valgrind_env_stuff)(Char** envp)
 Int VG_(waitpid)(Int pid, Int *status, Int options)
 {
 #  if defined(VGO_linux)
-   SysRes res = VG_(do_syscall4)(__NR_wait4, pid, (UWord)status, options, 0);
-   return res.isError ? -1 : res.res;
+   SysRes res = VG_(do_syscall4)(__NR_wait4,
+                                 pid, (UWord)status, options, 0);
+   return sr_isError(res) ? -1 : sr_Res(res);
+#  elif defined(VGO_darwin)
+   SysRes res = VG_(do_syscall4)(__NR_wait4_nocancel,
+                                 pid, (UWord)status, options, 0);
+   return sr_isError(res) ? -1 : sr_Res(res);
 #  elif defined(VGO_aix5)
    /* magic number 4 obtained by truss-ing a C program doing
       'waitpid'.  Note status and pid args opposite way round from
       POSIX. */
    SysRes res = VG_(do_syscall5)(__NR_AIX5_kwaitpid, 
                                  (UWord)status, pid, 4 | options,0,0);
-   if (0) VG_(printf)("waitpid: got 0x%lx 0x%lx\n", res.res, res.err);
-   return res.isError ? -1 : res.res;
+   if (0) VG_(printf)("waitpid: got 0x%lx 0x%lx\n", sr_Res(res), res.err);
+   return sr_isError(res) ? -1 : sr_Res(res);
 #  else
 #    error Unknown OS
 #  endif
@@ -286,7 +322,7 @@ void VG_(execv) ( Char* filename, Char** argv )
    res = VG_(do_syscall3)(__NR_execve,
                           (UWord)filename, (UWord)argv, (UWord)envp);
 
-   VG_(printf)("EXEC failed, errno = %ld\n", res.res);
+   VG_(printf)("EXEC failed, errno = %lld\n", (Long)sr_Err(res));
 }
 
 /* Return -1 if error, else 0.  NOTE does not indicate return code of
@@ -308,7 +344,6 @@ Int VG_(system) ( Char* cmd )
       VG_(exit)(1);
    } else {
       /* parent */
-      Int ir, zzz;
       /* We have to set SIGCHLD to its default behaviour in order that
          VG_(waitpid) works (at least on AIX).  According to the Linux
          man page for waitpid:
@@ -321,8 +356,10 @@ Int VG_(system) ( Char* cmd )
          set to ECHILD.  (The original POSIX standard left the
          behaviour of setting SIGCHLD to SIG_IGN unspecified.)
       */
-      struct vki_sigaction sa, saved_sa;
-      VG_(memset)( &sa, 0, sizeof(struct vki_sigaction) );
+      Int ir, zzz;
+      vki_sigaction_toK_t sa, sa2;
+      vki_sigaction_fromK_t saved_sa;
+      VG_(memset)( &sa, 0, sizeof(sa) );
       VG_(sigemptyset)(&sa.sa_mask);
       sa.ksa_handler = VKI_SIG_DFL;
       sa.sa_flags    = 0;
@@ -331,9 +368,9 @@ Int VG_(system) ( Char* cmd )
 
       zzz = VG_(waitpid)(pid, NULL, 0);
 
-      ir = VG_(sigaction)(VKI_SIGCHLD, &saved_sa, NULL);
+      VG_(convert_sigaction_fromK_to_toK)( &saved_sa, &sa2 );
+      ir = VG_(sigaction)(VKI_SIGCHLD, &sa2, NULL);
       vg_assert(ir == 0);
-
       return zzz == -1 ? -1 : 0;
    }
 }
@@ -350,9 +387,9 @@ Int VG_(getrlimit) (Int resource, struct vki_rlimit *rlim)
 #  ifdef __NR_ugetrlimit
    res = VG_(do_syscall2)(__NR_ugetrlimit, resource, (UWord)rlim);
 #  endif
-   if (res.isError && res.err == VKI_ENOSYS)
+   if (sr_isError(res) && sr_Err(res) == VKI_ENOSYS)
       res = VG_(do_syscall2)(__NR_getrlimit, resource, (UWord)rlim);
-   return res.isError ? -1 : res.res;
+   return sr_isError(res) ? -1 : sr_Res(res);
 }
 
 
@@ -362,7 +399,7 @@ Int VG_(setrlimit) (Int resource, const struct vki_rlimit *rlim)
    SysRes res;
    /* res = setrlimit( resource, rlim ); */
    res = VG_(do_syscall2)(__NR_setrlimit, resource, (UWord)rlim);
-   return res.isError ? -1 : res.res;
+   return sr_isError(res) ? -1 : sr_Res(res);
 }
 
 /* ---------------------------------------------------------------------
@@ -371,18 +408,10 @@ Int VG_(setrlimit) (Int resource, const struct vki_rlimit *rlim)
 
 Int VG_(gettid)(void)
 {
-#  if defined(VGO_aix5)
-   SysRes res;
-   Int    r;
-   vg_assert(__NR_AIX5__thread_self != __NR_AIX5_UNKNOWN);
-   res = VG_(do_syscall0)(__NR_AIX5__thread_self);
-   r = res.res;
-   return r;
-
-#  else
+#  if defined(VGO_linux)
    SysRes res = VG_(do_syscall0)(__NR_gettid);
 
-   if (res.isError && res.res == VKI_ENOSYS) {
+   if (sr_isError(res) && sr_Res(res) == VKI_ENOSYS) {
       Char pid[16];      
       /*
        * The gettid system call does not exist. The obvious assumption
@@ -400,13 +429,35 @@ Int VG_(gettid)(void)
 
       res = VG_(do_syscall3)(__NR_readlink, (UWord)"/proc/self",
                              (UWord)pid, sizeof(pid));
-      if (!res.isError && res.res > 0) {
-         pid[res.res] = '\0';
-         res.res = VG_(atoll)(pid);
+      if (!sr_isError(res) && sr_Res(res) > 0) {
+         Char* s;
+         pid[sr_Res(res)] = '\0';
+         res = VG_(mk_SysRes_Success)(  VG_(strtoll10)(pid, &s) );
+         if (*s != '\0') {
+            VG_(message)(Vg_DebugMsg, 
+               "Warning: invalid file name linked to by /proc/self: %s\n",
+               pid);
+         }
       }
    }
 
-   return res.res;
+   return sr_Res(res);
+
+#  elif defined(VGO_aix5)
+   SysRes res;
+   Int    r;
+   vg_assert(__NR_AIX5__thread_self != __NR_AIX5_UNKNOWN);
+   res = VG_(do_syscall0)(__NR_AIX5__thread_self);
+   r = sr_Res(res);
+   return r;
+
+#  elif defined(VGO_darwin)
+   // Darwin's gettid syscall is something else.
+   // Use Mach thread ports for lwpid instead.
+   return mach_thread_self();
+
+#  else
+#    error "Unknown OS"
 #  endif
 }
 
@@ -414,32 +465,32 @@ Int VG_(gettid)(void)
 Int VG_(getpid) ( void )
 {
    /* ASSUMES SYSCALL ALWAYS SUCCEEDS */
-   return VG_(do_syscall0)(__NR_getpid) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_getpid) );
 }
 
 Int VG_(getpgrp) ( void )
 {
    /* ASSUMES SYSCALL ALWAYS SUCCEEDS */
-   return VG_(do_syscall0)(__NR_getpgrp) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_getpgrp) );
 }
 
 Int VG_(getppid) ( void )
 {
    /* ASSUMES SYSCALL ALWAYS SUCCEEDS */
-   return VG_(do_syscall0)(__NR_getppid) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_getppid) );
 }
 
 Int VG_(geteuid) ( void )
 {
    /* ASSUMES SYSCALL ALWAYS SUCCEEDS */
 #  if defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
-   return VG_(do_syscall1)(__NR_AIX5_getuidx, 1) . res;
+   return sr_Res( VG_(do_syscall1)(__NR_AIX5_getuidx, 1) );
 #  elif defined(__NR_geteuid32)
    // We use the 32-bit version if it's supported.  Otherwise, IDs greater
    // than 65536 cause problems, as bug #151209 showed.
-   return VG_(do_syscall0)(__NR_geteuid32) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_geteuid32) );
 #  else
-   return VG_(do_syscall0)(__NR_geteuid) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_geteuid) );
 #  endif
 }
 
@@ -447,13 +498,13 @@ Int VG_(getegid) ( void )
 {
    /* ASSUMES SYSCALL ALWAYS SUCCEEDS */
 #  if defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
-   return VG_(do_syscall1)(__NR_AIX5_getgidx, 1) . res;
+   return sr_Res( VG_(do_syscall1)(__NR_AIX5_getgidx, 1) );
 #  elif defined(__NR_getegid32)
    // We use the 32-bit version if it's supported.  Otherwise, IDs greater
    // than 65536 cause problems, as bug #151209 showed.
-   return VG_(do_syscall0)(__NR_getegid32) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_getegid32) );
 #  else
-   return VG_(do_syscall0)(__NR_getegid) . res;
+   return sr_Res( VG_(do_syscall0)(__NR_getegid) );
 #  endif
 }
 
@@ -470,21 +521,22 @@ Int VG_(getgroups)( Int size, UInt* list )
    if (size < 0) return -1;
    if (size > 64) size = 64;
    sres = VG_(do_syscall2)(__NR_getgroups, size, (Addr)list16);
-   if (sres.isError)
+   if (sr_isError(sres))
       return -1;
-   if (sres.res > size)
+   if (sr_Res(sres) > size)
       return -1;
-   for (i = 0; i < sres.res; i++)
+   for (i = 0; i < sr_Res(sres); i++)
       list[i] = (UInt)list16[i];
-   return sres.res;
+   return sr_Res(sres);
 
 #  elif defined(VGP_amd64_linux) || defined(VGP_ppc64_linux) \
-        || defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+        || defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5) \
+        || defined(VGO_darwin)
    SysRes sres;
    sres = VG_(do_syscall2)(__NR_getgroups, size, (Addr)list);
-   if (sres.isError)
+   if (sr_isError(sres))
       return -1;
-   return sres.res;
+   return sr_Res(sres);
 
 #  else
 #     error "VG_(getgroups): needs implementation on this platform"
@@ -499,9 +551,9 @@ Int VG_(ptrace) ( Int request, Int pid, void *addr, void *data )
 {
    SysRes res;
    res = VG_(do_syscall4)(__NR_ptrace, request, pid, (UWord)addr, (UWord)data);
-   if (res.isError)
+   if (sr_isError(res))
       return -1;
-   return res.res;
+   return sr_Res(res);
 }
 
 /* ---------------------------------------------------------------------
@@ -510,11 +562,27 @@ Int VG_(ptrace) ( Int request, Int pid, void *addr, void *data )
 
 Int VG_(fork) ( void )
 {
+#  if defined(VGO_linux) || defined(VGO_aix5)
    SysRes res;
    res = VG_(do_syscall0)(__NR_fork);
-   if (res.isError)
+   if (sr_isError(res))
       return -1;
-   return res.res;
+   return sr_Res(res);
+
+#  elif defined(VGO_darwin)
+   SysRes res;
+   res = VG_(do_syscall0)(__NR_fork); /* __NR_fork is UX64 */
+   if (sr_isError(res))
+      return -1;
+   /* on success: wLO = child pid; wHI = 1 for child, 0 for parent */
+   if (sr_ResHI(res) != 0) {
+      return 0;  /* this is child: return 0 instead of child pid */
+   }
+   return sr_Res(res);
+
+#  else
+#    error "Unknown OS"
+#  endif
 }
 
 /* ---------------------------------------------------------------------
@@ -527,7 +595,22 @@ UInt VG_(read_millisecond_timer) ( void )
    static ULong base = 0;
    ULong  now;
 
-#  if defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+#  if defined(VGO_linux)
+   { SysRes res;
+     struct vki_timespec ts_now;
+     res = VG_(do_syscall2)(__NR_clock_gettime, VKI_CLOCK_MONOTONIC,
+                            (UWord)&ts_now);
+     if (sr_isError(res) == 0) {
+        now = ts_now.tv_sec * 1000000ULL + ts_now.tv_nsec / 1000;
+     } else {
+       struct vki_timeval tv_now;
+       res = VG_(do_syscall2)(__NR_gettimeofday, (UWord)&tv_now, (UWord)NULL);
+       vg_assert(! sr_isError(res));
+       now = tv_now.tv_sec * 1000000ULL + tv_now.tv_usec;
+     }
+   }
+
+#  elif defined(VGO_aix5)
    /* AIX requires a totally different implementation since
       sys_gettimeofday doesn't exist.  We use the POWER real-time
       register facility.  This will SIGILL on PowerPC 970 on AIX,
@@ -545,31 +628,31 @@ UInt VG_(read_millisecond_timer) ( void )
    vg_assert(nsec < 1000*1000*1000);
    now  = ((ULong)sec1) * 1000000ULL;
    now += (ULong)(nsec / 1000);
-#  else
 
-   struct vki_timespec ts_now;
-   SysRes res;
-   res = VG_(do_syscall2)(__NR_clock_gettime, VKI_CLOCK_MONOTONIC,
-                          (UWord)&ts_now);
-   if (res.isError == 0)
-   {
-     now = ts_now.tv_sec * 1000000ULL + ts_now.tv_nsec / 1000;
-   }
-   else
-   {
-     struct vki_timeval tv_now;
+#  elif defined(VGO_darwin)
+   // Weird: it seems that gettimeofday() doesn't fill in the timeval, but
+   // rather returns the tv_sec as the low 32 bits of the result and the
+   // tv_usec as the high 32 bits of the result.  (But the timeval cannot be
+   // NULL!)  See bug 200990.
+   { SysRes res;
+     struct vki_timeval tv_now = { 0, 0 };
      res = VG_(do_syscall2)(__NR_gettimeofday, (UWord)&tv_now, (UWord)NULL);
-     vg_assert(! res.isError);
-     now = tv_now.tv_sec * 1000000ULL + tv_now.tv_usec;
+     vg_assert(! sr_isError(res));
+     now = sr_Res(res) * 1000000ULL + sr_ResHI(res);
    }
+
+#  else
+#    error "Unknown OS"
 #  endif
-   
+
+   /* COMMON CODE */  
    if (base == 0)
       base = now;
 
    return (now - base) / 1000;
 }
 
+
 /* ---------------------------------------------------------------------
    atfork()
    ------------------------------------------------------------------ */
@@ -634,6 +717,7 @@ void VG_(do_atfork_child)(ThreadId tid)
          (*atforks[i].child)(tid);
 }
 
+
 /*--------------------------------------------------------------------*/
 /*--- end                                                          ---*/
 /*--------------------------------------------------------------------*/
