@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward
+   Copyright (C) 2000-2009 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -50,8 +50,9 @@ extern Bool ML_(client_signal_OK)(Int sigNo);
 
 // Return true if we're allowed to use or create this fd.
 extern
-Bool ML_(fd_allowed)(Int fd, const Char *syscallname, ThreadId tid, Bool soft);
+Bool ML_(fd_allowed)(Int fd, const Char *syscallname, ThreadId tid, Bool isNewFD);
 
+extern void ML_(record_fd_open_named)          (ThreadId tid, Int fd);
 extern void ML_(record_fd_open_nameless)       (ThreadId tid, Int fd);
 extern void ML_(record_fd_open_with_given_name)(ThreadId tid, Int fd,
                                                 char *pathname);
@@ -61,13 +62,28 @@ extern void ML_(record_fd_open_with_given_name)(ThreadId tid, Int fd,
 extern
 Bool ML_(do_sigkill)(Int pid, Int tgid);
 
-/* So that it can be seen from syswrap-x86-linux.c. */
-/* When a client mmap has been successfully done, both aspacem and the
-   tool need to be notified of the new mapping.  Hence this fn. */
+/* When a client mmap or munmap has been successfully done, both the core 
+   and the tool need to be notified of the new mapping.  Hence this fn. */
+extern void 
+ML_(notify_core_and_tool_of_mmap) ( Addr a, SizeT len, UInt prot, 
+                                    UInt mm_flags, Int fd, Off64T offset );
+extern void 
+ML_(notify_core_and_tool_of_munmap) ( Addr a, SizeT len );
+extern void 
+ML_(notify_core_and_tool_of_mprotect) ( Addr a, SizeT len, Int prot );
+
+extern void
+ML_(buf_and_len_pre_check) ( ThreadId tid, Addr buf_p, Addr buflen_p,
+                             Char* buf_s, Char* buflen_s );
+extern void
+ML_(buf_and_len_post_check) ( ThreadId tid, SysRes res,
+                              Addr buf_p, Addr buflen_p, Char* s );
+
+/* PRE and POST for unknown ioctls based on ioctl request encoding */
 extern 
-void 
-ML_(notify_aspacem_and_tool_of_mmap) ( Addr a, SizeT len, UInt prot, 
-                                       UInt mm_flags, Int fd, ULong offset );
+void ML_(PRE_unknown_ioctl)(ThreadId tid, UWord request, UWord arg);
+extern 
+void ML_(POST_unknown_ioctl)(ThreadId tid, UInt res, UWord request, UWord arg);
 
 
 DECL_TEMPLATE(generic, sys_ni_syscall);            // * P -- unimplemented
@@ -96,7 +112,6 @@ DECL_TEMPLATE(generic, sys_mkdir);
 DECL_TEMPLATE(generic, sys_rmdir);
 DECL_TEMPLATE(generic, sys_dup);
 DECL_TEMPLATE(generic, sys_times);
-DECL_TEMPLATE(generic, sys_fcntl);        // POSIX (but complicated)
 DECL_TEMPLATE(generic, sys_setpgid);
 DECL_TEMPLATE(generic, sys_umask);
 DECL_TEMPLATE(generic, sys_dup2);
@@ -170,16 +185,13 @@ DECL_TEMPLATE(generic, sys_newfstat);              // * P (SVr4,BSD4.3)
 
 // For the remainder, not really sure yet
 DECL_TEMPLATE(generic, sys_ptrace);                // (x86?) (almost-P)
-DECL_TEMPLATE(generic, sys_sigsuspend);            // POSIX, but L (proto varies across archs)
 DECL_TEMPLATE(generic, sys_setrlimit);             // SVr4, 4.3BSD
-DECL_TEMPLATE(generic, sys_ioctl);                 // x86? (various)
 DECL_TEMPLATE(generic, sys_old_getrlimit);         // SVr4, 4.3BSD L?
 DECL_TEMPLATE(generic, sys_statfs);                // * L?
 DECL_TEMPLATE(generic, sys_fstatfs);               // * L?
 DECL_TEMPLATE(generic, sys_iopl);                  // (x86/amd64) L
 DECL_TEMPLATE(generic, sys_ipc);                   // (x86) L
 DECL_TEMPLATE(generic, sys_newuname);              // * P
-DECL_TEMPLATE(generic, sys_init_module);           // * L?
 DECL_TEMPLATE(generic, sys_pread64);               // * (Unix98?)
 DECL_TEMPLATE(generic, sys_pwrite64);              // * (Unix98?)
 DECL_TEMPLATE(generic, sys_sigaltstack);           // (x86) (XPG4-UNIX)
@@ -191,8 +203,6 @@ DECL_TEMPLATE(generic, sys_ftruncate64);           // %% (P?)
 DECL_TEMPLATE(generic, sys_lchown);                // * (L?)
 DECL_TEMPLATE(generic, sys_mincore);               // * L?
 DECL_TEMPLATE(generic, sys_getdents64);            // * (SVr4,SVID?)
-DECL_TEMPLATE(generic, sys_fcntl64);               // * P?
-DECL_TEMPLATE(generic, sys_lookup_dcookie);        // (*/32/64) L
 DECL_TEMPLATE(generic, sys_statfs64);              // * (?)
 DECL_TEMPLATE(generic, sys_fstatfs64);             // * (?)
 
@@ -221,8 +231,6 @@ extern void   ML_(generic_PRE_sys_recv)         ( TId, UW, UW, UW );
 extern void   ML_(generic_POST_sys_recv)        ( TId, UW, UW, UW, UW );
 extern void   ML_(generic_PRE_sys_connect)      ( TId, UW, UW, UW );
 extern void   ML_(generic_PRE_sys_setsockopt)   ( TId, UW, UW, UW, UW, UW );
-extern void   ML_(generic_PRE_sys_getsockopt)   ( TId, UW, UW, UW, UW, UW );
-extern void   ML_(generic_POST_sys_getsockopt)  ( TId, SR, UW, UW, UW, UW, UW );
 extern void   ML_(generic_PRE_sys_getsockname)  ( TId, UW, UW, UW );
 extern void   ML_(generic_POST_sys_getsockname) ( TId, SR, UW, UW, UW );
 extern void   ML_(generic_PRE_sys_getpeername)  ( TId, UW, UW, UW );
@@ -244,6 +252,26 @@ extern void   ML_(generic_POST_sys_shmctl)      ( TId, UW, UW, UW, UW );
 
 extern SysRes ML_(generic_PRE_sys_mmap)         ( TId, UW, UW, UW, UW, UW, Off64T );
 
+#define PRE_timeval_READ(zzname, zzarg)                        \
+   do {                                                        \
+      struct vki_timeval *zztv = (struct vki_timeval *)zzarg;  \
+      PRE_FIELD_READ(zzname, zztv->tv_sec);                    \
+      PRE_FIELD_READ(zzname, zztv->tv_usec);                   \
+   } while (0)
+#define PRE_timeval_WRITE(zzname, zzarg)                       \
+   do {                                                        \
+      struct vki_timeval *zztv = (struct vki_timeval *)zzarg;  \
+      PRE_FIELD_WRITE(zzname, zztv->tv_sec);                   \
+      PRE_FIELD_WRITE(zzname, zztv->tv_usec);                  \
+   } while (0)
+#define POST_timeval_WRITE(zzarg)                              \
+   do {                                                        \
+      struct vki_timeval *zztv = (struct vki_timeval *)zzarg;  \
+      POST_FIELD_WRITE(zztv->tv_sec);                          \
+      POST_FIELD_WRITE(zztv->tv_usec);                         \
+   } while (0)
+
+
 #undef TId
 #undef UW
 #undef SR
