@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward
+   Copyright (C) 2000-2009 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -67,8 +67,8 @@ typedef
 typedef
    enum { 
       VgSrc_None,	 /* not exiting yet */
-      VgSrc_ExitSyscall, /* client called exit().  This is the normal
-                            route out. */
+      VgSrc_ExitThread,  /* just this thread is exiting */
+      VgSrc_ExitProcess, /* entire process is exiting */
       VgSrc_FatalSig	 /* Killed by the default action of a fatal
 			    signal */
    }
@@ -81,46 +81,208 @@ typedef
    typedef VexGuestAMD64State VexGuestArchState;
 #elif defined(VGA_ppc32)
    typedef VexGuestPPC32State VexGuestArchState;
+#elif defined(VGA_ppc64)
+   typedef VexGuestPPC64State VexGuestArchState;
 #else
 #  error Unknown architecture
 #endif
 
+/* Forward declarations */
+struct SyscallStatus;
+struct SyscallArgs;
 
+/* Architecture-specific thread state */
 typedef 
    struct {
       /* --- BEGIN vex-mandated guest state --- */
 
-      /* Saved machine context. */
-      VexGuestArchState vex;
+      /* Note that for code generation reasons, we require that the
+         guest state area, its two shadows, and the spill area, are
+         16-aligned and have 16-aligned sizes, and there are no holes
+         in between.  This is checked by do_pre_run_checks() in
+         scheduler.c. */
 
-      /* Saved shadow context. */
-      VexGuestArchState vex_shadow;
+      /* Saved machine context. */
+      VexGuestArchState vex __attribute__((aligned(16)));
+
+      /* Saved shadow context (2 copies). */
+      VexGuestArchState vex_shadow1 __attribute__((aligned(16)));
+      VexGuestArchState vex_shadow2 __attribute__((aligned(16)));
 
       /* Spill area. */
-      UChar vex_spill[LibVEX_N_SPILL_BYTES];
+      UChar vex_spill[LibVEX_N_SPILL_BYTES] __attribute__((aligned(16)));
 
       /* --- END vex-mandated guest state --- */
    } 
    ThreadArchState;
 
-/* OS-specific thread state */
-typedef struct {
-   /* who we are */
-   Int	lwpid;			// PID of kernel task
-   Int	threadgroup;		// thread group id
 
-   ThreadId parent;		// parent tid (if any)
+/* OS-specific thread state.  IMPORTANT: if you add fields to this,
+   you _must_ add code to os_state_clear() to initialise those
+   fields. */
+typedef
+   struct {
+      /* who we are */
+      Int lwpid;        // PID of kernel task  (Darwin: Mach thread)
+      Int threadgroup;  // thread group id
 
-   /* runtime details */
-   Addr valgrind_stack_base;    // Valgrind's stack (VgStack*)
-   Addr valgrind_stack_init_SP; // starting value for SP
+      ThreadId parent;  // parent tid (if any)
 
-   /* exit details */
-   Int  exitcode;		// in the case of exitgroup, set by someone else
-   Int  fatalsig;		// fatal signal
-} os_thread_t;
+      /* runtime details */
+      Addr valgrind_stack_base;    // Valgrind's stack (VgStack*)
+      Addr valgrind_stack_init_SP; // starting value for SP
+
+      /* exit details */
+      Word exitcode; // in the case of exitgroup, set by someone else
+      Int  fatalsig; // fatal signal
+
+#     if defined(VGO_aix5)
+      /* AIX specific fields to make thread cancellation sort-of work */
+      /* What is this thread's current cancellation state a la
+         POSIX (deferred vs async, enable vs disabled) ? */
+      Bool cancel_async;   // current cancel mode (async vs deferred)
+      Bool cancel_disabled; // cancellation disabled?
+      /* What's happened so far? */
+      enum { Canc_NoRequest=0, // no cancellation requested
+             Canc_Requested=1, // requested but not actioned
+             Canc_Actioned=2 } // requested and actioned
+           cancel_progress;
+      /* Initial state is False, False, Canc_Normal. */
+#     endif
+
+#     if defined(VGO_darwin)
+      // Mach trap POST handler as chosen by PRE
+      void (*post_mach_trap_fn)(ThreadId tid,
+                                struct SyscallArgs *, struct SyscallStatus *);
+    
+      // This thread's pthread
+      Addr pthread;
+    
+      // Argument passed when thread started
+      Addr func_arg;
+
+      // Synchronization between child thread and parent thread's POST wrapper
+      semaphore_t child_go;
+      semaphore_t child_done;
+
+      // Workqueue re-entry 
+      // (setjmp in PRE(workq_ops), longjmp in wqthread_hijack)
+      // DDD: JRS fixme: this comment is no longer correct; wq_jmpbuf is
+      // never used, and there is no such setjmp or longjmp pair.
+      // I guess we could leave wq_jmpbuf_valid in place though, since
+      // it does allow for an assertion in ML_(wqthread_continue_NORETURN).
+      Bool wq_jmpbuf_valid;
+      //jmp_buf wq_jmpbuf;
+
+      // Values saved from transient Mach RPC messages
+      Addr remote_port;  // destination for original message
+      Int msgh_id;       // outgoing message id
+      union {
+         struct {
+            Addr port;
+         } mach_port;
+         struct {
+            Int right;
+         } mach_port_allocate;
+         struct {
+            Addr port;
+            Int right;
+            Int delta;
+         } mach_port_mod_refs;
+         struct {
+            Addr task;
+            Addr name;
+            Int disposition;
+         } mach_port_insert_right;
+         struct {
+            Addr size;
+            int flags;
+         } vm_allocate;
+         struct {
+            Addr address;
+            Addr size;
+         } vm_deallocate;
+         struct {
+            Addr src;
+            Addr dst;
+            Addr size;
+         } vm_copy;
+         struct {
+            Addr address;
+            Addr size;
+            int set_maximum;
+            UWord new_protection;
+         } vm_protect;
+         struct {
+            Addr addr;
+            SizeT size;
+         } vm_read;
+         struct {
+            ULong addr;
+            ULong size;
+         } mach_vm_read;
+         struct {
+            Addr addr;
+            SizeT size;
+            Addr data;
+         } vm_read_overwrite;
+         struct {
+            Addr size;
+            int copy;
+            UWord protection;
+         } vm_map;
+         struct {
+            Addr size;
+         } vm_remap;
+         struct {
+            ULong size;
+            int flags;
+         } mach_vm_allocate;
+         struct {
+            ULong address;
+            ULong size;
+         } mach_vm_deallocate;
+         struct {
+            ULong address;
+            ULong size;
+            int set_maximum;
+            unsigned int new_protection;
+         } mach_vm_protect;
+         struct {
+            ULong size;
+            int copy;
+            UWord protection;
+         } mach_vm_map;
+         struct {
+            Addr thread;
+            UWord flavor;
+         } thread_get_state;
+         struct {
+            Addr address;
+         } io_connect_unmap_memory;
+         struct {
+            int which_port;
+         } task_get_special_port;
+         struct {
+            char *service_name;
+         } bootstrap_look_up;
+         struct {
+            vki_size_t size;
+         } WindowServer_29828;
+         struct {
+            Int access_rights;
+         } WindowServer_29831;
+         struct {
+            char *path;
+         } io_registry_entry_from_path;
+      } mach_args;
+#     endif
+
+   }
+   ThreadOSstate;
 
 
+/* Overall thread state */
 typedef struct {
    /* ThreadId == 0 (and hence vg_threads[0]) is NEVER USED.
       The thread identity is simply the index in vg_threads[].
@@ -169,31 +331,25 @@ typedef struct {
       only then is the old one deallocated and a new one
       allocated. 
 
-      For the main thread (threadid == 0), this mechanism doesn't
+      For the main thread (threadid == 1), this mechanism doesn't
       apply.  We don't know the size of the stack since we didn't
       allocate it, and furthermore we never reallocate it. */
 
-   /* The allocated size of this thread's stack (permanently zero
-      if this is ThreadId == 0, since we didn't allocate its stack) */
+   /* The allocated size of this thread's stack */
    SizeT client_stack_szB;
 
    /* Address of the highest legitimate word in this stack.  This is
       used for error messages only -- not critical for execution
       correctness.  Is is set for all stacks, specifically including
-      ThreadId == 0 (the main thread). */
+      ThreadId == 1 (the main thread). */
    Addr client_stack_highest_word;
 
    /* Alternate signal stack */
    vki_stack_t altstack;
 
    /* OS-specific thread state */
-   os_thread_t os_state;
+   ThreadOSstate os_state;
 
-   /* Used in the syscall handlers.  Set to True to indicate that the
-      PRE routine for a syscall has set the syscall result already and
-      so the syscall does not need to be handed to the kernel. */
-   Bool syscall_result_set;
-   
    /* Per-thread jmp_buf to resume scheduler after a signal */
    Bool    sched_jmpbuf_valid;
    jmp_buf sched_jmpbuf;
@@ -214,6 +370,7 @@ extern ThreadState VG_(threads)[VG_N_THREADS];
 // to write to this.
 extern ThreadId VG_(running_tid);
 
+
 /*------------------------------------------------------------*/
 /*--- Basic operations on the thread table.                ---*/
 /*------------------------------------------------------------*/
@@ -236,9 +393,12 @@ extern Bool VG_(is_exiting)(ThreadId tid);
 /* Return the number of non-dead Threads */
 extern Int VG_(count_living_threads)(void);
 
+/* Return the number of threads in VgTs_Runnable state */
+extern Int VG_(count_runnable_threads)(void);
+
 /* Given an LWP id (ie, real kernel thread id), find the corresponding
    ThreadId */
-extern ThreadId VG_(get_lwp_tid)(Int lwpid);
+extern ThreadId VG_(lwpid_to_vgtid)(Int lwpid);
 
 #endif   // __PUB_CORE_THREADSTATE_H
 
