@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2008 Julian Seward 
+   Copyright (C) 2000-2009 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -37,7 +37,7 @@
 //--------------------------------------------------------------------
 
 /*------------------------------------------------------------*/
-/*--- Loading ELF files                                    ---*/
+/*--- Loading files                                        ---*/
 /*------------------------------------------------------------*/
 
 // Info needed to load and run a program.  IN/INOUT/OUT refers to the
@@ -49,9 +49,18 @@ typedef
       Addr exe_base;     // INOUT: lowest (allowed) address of exe
       Addr exe_end;      // INOUT: highest (allowed) address
 
+#if !defined(VGO_darwin)
       Addr phdr;         // OUT: address phdr was mapped at
       Int  phnum;        // OUT: number of phdrs
       Addr interp_base;  // OUT: where interpreter (ld.so) was mapped
+#else
+      Addr  stack_start;      // OUT: address of start of stack segment (hot)
+      Addr  stack_end;        // OUT: address of end of stack segment (cold)
+      Addr  text;             // OUT: address of executable's Mach header
+      Bool  dynamic;          // OUT: False iff executable is static
+      char* executable_path;  // OUT: path passed to execve()
+#endif
+
       Addr entry;        // OUT: entrypoint in main executable
       Addr init_ip;      // OUT: address of first instruction to execute
       Addr brkbase;      // OUT: base address of brk segment
@@ -67,7 +76,7 @@ typedef
 
 // Do a number of appropriate checks to see if the file looks executable by
 // the kernel: ie. it's a file, it's readable and executable, and it's in
-// either ELF or "#!" format.  On success, 'out_fd' gets the fd of the file
+// either binary or "#!" format.  On success, 'out_fd' gets the fd of the file
 // if it's non-NULL.  Otherwise the fd is closed.
 extern SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd,
                                   Bool allow_setuid);
@@ -78,21 +87,6 @@ extern SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd,
 // the program.
 extern Int VG_(do_exec)(const HChar* exe, ExeInfo* info);
 
-/*------------------------------------------------------------*/
-/*--- Finding and dealing with auxv                        ---*/
-/*------------------------------------------------------------*/
-
-struct ume_auxv
-{
-   Word a_type;
-   union {
-      void *a_ptr;
-      Word a_val;
-   } u;
-};
-
-extern struct ume_auxv *VG_(find_auxv)(UWord* orig_esp);
-
 #endif /* __PUB_CORE_UME_H */
 
 /*--------------------------------------------------------------------*/
