@@ -1,18 +1,17 @@
 
 /*--------------------------------------------------------------------*/
-/*--- Top level for kernel interface declarations.                 ---*/
-/*---                                         pub_core_vkiscnums.h ---*/
+/*--- Syscall numbers and related operations. pub_core_vkiscnums.h ---*/
 /*--------------------------------------------------------------------*/
 
 /*
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward
+   Copyright (C) 2000-2009 Julian Seward
       jseward@acm.org
-   Copyright (C) 2005-2007 Nicholas Nethercote
+   Copyright (C) 2005-2009 Nicholas Nethercote
       njn@valgrind.org
-   Copyright (C) 2006-2007 OpenWorks LLP
+   Copyright (C) 2006-2009 OpenWorks LLP
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -36,27 +35,52 @@
 #ifndef __PUB_CORE_VKISCNUMS_H
 #define __PUB_CORE_VKISCNUMS_H
 
+//--------------------------------------------------------------------
+// PURPOSE: This module holds all the syscall numbers, and a handful of
+// operations relating to them.
+//
+// On Linux they are a bunch of #define'd constants of the form
+// __NR_name, and this file must contain nothing else, since it will
+// be included in assembly code (m_trampoline.S).
+//
+// On AIX the __NR_name consts are renamings of global variables which
+// tell us the number for each syscall.  This elaboration is necessary
+// because on AIX the syscall numbers are not constant; they can be
+// different for each process (in principle; in practice they rarely
+// change).  32- and 64-bit AIX5 share a common "implementation".
+//
+// On Darwin the __NR_name consts are #define'd constants which are
+// encoded using various macros.  32- and 64-bit Darwin share a common
+// "implementation" also.
+//
+// Note that most of the actual code for this module is in include/vki/, but
+// you should not directly #include any file in include/vki; instead #include
+// only one of pub_{core,tool}_vkiscnums{,_asm}.h.
+//--------------------------------------------------------------------
+
 /* Most unfortunately, all the kernel decls are visible to tools.  Not
    really necessary, but to avoid this would require some tedious
    refactoring of the sources.  Anyway, we live with this kludge, and
    that means the only thing to be done here is ... */
 
+#include "pub_core_vkiscnums_asm.h"
 #include "pub_tool_vkiscnums.h"
 
 
-/* Make it possible to include this file in assembly sources. */
-#if !defined(VG_IN_ASSEMBLY_SOURCE)
+#if defined(VGO_linux)
+   // Nothing
 
-#if defined(VGO_aix5)
+#elif defined(VGO_aix5)
 /* Bind the given syscall name to the given number.  Returns True if
    successful, False if the name is unknown. */
 extern Bool VG_(aix5_register_syscall)( Int, UChar* );
-/* Look up in said binding later, for the purposes of making error
-   messages. */
-extern UChar* VG_(aix5_sysno_to_sysname)( Int sysno );
-#endif
 
-#endif /* !defined(VG_IN_ASSEMBLY_SOURCE) */
+#elif defined(VGO_darwin)
+   // Nothing
+
+#else
+#  error Unknown OS
+#endif
 
 #endif // __PUB_CORE_VKISCNUMS_H
 
