@@ -9,7 +9,7 @@
    This file is part of Ptrcheck, a Valgrind tool for checking pointer
    use in programs.
 
-   Copyright (C) 2008-2008 OpenWorks Ltd
+   Copyright (C) 2008-2009 OpenWorks Ltd
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -223,16 +223,16 @@ static Word StackBlocks__cmp ( XArray* fb1s, XArray* fb2s )
 static void pp_StackBlocks ( XArray* sbs )
 {
    Word i, n = VG_(sizeXA)( sbs );
-   VG_(message)(Vg_DebugMsg, "<<< STACKBLOCKS" );
+   VG_(message)(Vg_DebugMsg, "<<< STACKBLOCKS\n" );
    for (i = 0; i < n; i++) {
       StackBlock* sb = (StackBlock*)VG_(indexXA)( sbs, i );
       VG_(message)(Vg_DebugMsg,
-         "   StackBlock{ off %ld szB %lu spRel:%c isVec:%c \"%s\" }",
+         "   StackBlock{ off %ld szB %lu spRel:%c isVec:%c \"%s\" }\n",
          sb->base, sb->szB, sb->spRel ? 'Y' : 'N',
          sb->isVec ? 'Y' : 'N', &sb->name[0] 
       );
    }
-   VG_(message)(Vg_DebugMsg, ">>> STACKBLOCKS" );
+   VG_(message)(Vg_DebugMsg, ">>> STACKBLOCKS\n" );
 }
 
 
@@ -345,12 +345,12 @@ static XArray* /* of StackBlock */
           if (moans > 0 && !VG_(clo_xml)) {
              moans--;
              VG_(message)(Vg_UserMsg, "Warning: bogus DWARF3 info: "
-                                      "overlapping stack blocks");
+                                      "overlapping stack blocks\n");
              if (VG_(clo_verbosity) >= 2)
                 pp_StackBlocks(orig);
              if (moans == 0)
                 VG_(message)(Vg_UserMsg, "Further instances of this "
-                                         "message will not be shown" );
+                                         "message will not be shown\n" );
           }
           VG_(dropTailXA)( orig, VG_(sizeXA)( orig ));
           break;
@@ -740,7 +740,7 @@ static void add_block_to_GlobalTree (
    if (already_present && moans > 0 && !VG_(clo_xml)) {
       moans--;
       VG_(message)(Vg_UserMsg, "Warning: bogus DWARF3 info: "
-                               "overlapping global blocks");
+                               "overlapping global blocks\n");
       if (VG_(clo_verbosity) >= 2) {
          GlobalTree__pp( gitree,
                          "add_block_to_GlobalTree: non-exact duplicate" );
@@ -750,7 +750,7 @@ static void add_block_to_GlobalTree (
       }
       if (moans == 0)
          VG_(message)(Vg_UserMsg, "Further instances of this "
-                                  "message will not be shown" );
+                                  "message will not be shown\n" );
    }
    /* tl_assert(!already_present); */
 }
@@ -1830,7 +1830,7 @@ void shadowStack_new_frame ( ThreadId tid,
          if (0 && (sb || gb))
             VG_(message)(Vg_DebugMsg, 
                          "exp-sgcheck: new max tree sizes: "
-                         "StackTree %ld, GlobalTree %ld",
+                         "StackTree %ld, GlobalTree %ld\n",
                          stats__max_sitree_size, stats__max_gitree_size );
       }
    } else {
@@ -2012,12 +2012,16 @@ struct _SGEnv {
       we basically can't really handle properly; and so we ignore all
       but the first ref. */
    Bool firstRef;
+   /* READONLY */
+   IRTemp (*newIRTemp_cb)(IRType,void*);
+   void* newIRTemp_opaque;
 };
 
 
 /* --- Helper fns for instrumentation --- */
 
-static IRTemp gen_Get_SP ( IRSB*           bbOut,
+static IRTemp gen_Get_SP ( struct _SGEnv*  sge,
+                           IRSB*           bbOut,
                            VexGuestLayout* layout,
                            Int             hWordTy_szB )
 {
@@ -2029,12 +2033,13 @@ static IRTemp gen_Get_SP ( IRSB*           bbOut,
    tl_assert(hWordTy_szB == layout->sizeof_SP);
    sp_type = layout->sizeof_SP == 8 ? Ity_I64 : Ity_I32;
    sp_expr = IRExpr_Get( layout->offset_SP, sp_type );
-   sp_temp = newIRTemp( bbOut->tyenv, sp_type );
+   sp_temp = sge->newIRTemp_cb( sp_type, sge->newIRTemp_opaque );
    addStmtToIRSB( bbOut, IRStmt_WrTmp( sp_temp, sp_expr ) );
    return sp_temp;
 }
 
-static IRTemp gen_Get_FP ( IRSB*           bbOut,
+static IRTemp gen_Get_FP ( struct _SGEnv*  sge,
+                           IRSB*           bbOut,
                            VexGuestLayout* layout,
                            Int             hWordTy_szB )
 {
@@ -2046,12 +2051,13 @@ static IRTemp gen_Get_FP ( IRSB*           bbOut,
    tl_assert(hWordTy_szB == layout->sizeof_SP);
    fp_type = layout->sizeof_FP == 8 ? Ity_I64 : Ity_I32;
    fp_expr = IRExpr_Get( layout->offset_FP, fp_type );
-   fp_temp = newIRTemp( bbOut->tyenv, fp_type );
+   fp_temp = sge->newIRTemp_cb( fp_type, sge->newIRTemp_opaque );
    addStmtToIRSB( bbOut, IRStmt_WrTmp( fp_temp, fp_expr ) );
    return fp_temp;
 }
 
-static void instrument_mem_access ( IRSB*   bbOut, 
+static void instrument_mem_access ( struct _SGEnv* sge,
+                                    IRSB*   bbOut, 
                                     IRExpr* addr,
                                     Int     szB,
                                     Bool    isStore,
@@ -2088,8 +2094,8 @@ static void instrument_mem_access ( IRSB*   bbOut,
    /* Generate a call to "helperc__mem_access", passing:
          addr current_SP current_FP szB curr_IP frameBlocks
    */
-   { IRTemp t_SP = gen_Get_SP( bbOut, layout, hWordTy_szB );
-     IRTemp t_FP = gen_Get_FP( bbOut, layout, hWordTy_szB );
+   { IRTemp t_SP = gen_Get_SP( sge, bbOut, layout, hWordTy_szB );
+     IRTemp t_FP = gen_Get_FP( sge, bbOut, layout, hWordTy_szB );
      IRExpr** args
         = mkIRExprVec_6( addr,
                          IRExpr_RdTmp(t_SP),
@@ -2100,7 +2106,7 @@ static void instrument_mem_access ( IRSB*   bbOut,
      IRDirty* di
         = unsafeIRDirty_0_N( 3/*regparms*/, 
                              "helperc__mem_access", 
-                             VG_(fnptr_to_fnentry)( &helperc__mem_access ),
+                            VG_(fnptr_to_fnentry)( &helperc__mem_access ),
                              args );
 
      addStmtToIRSB( bbOut, IRStmt_Dirty(di) );
@@ -2110,14 +2116,17 @@ static void instrument_mem_access ( IRSB*   bbOut,
 
 /* --- Instrumentation main (4 fns) --- */
 
-struct _SGEnv *  sg_instrument_init ( void )
+struct _SGEnv * sg_instrument_init ( IRTemp (*newIRTemp_cb)(IRType,void*),
+                                     void* newIRTemp_opaque )
 {
    struct _SGEnv * env = sg_malloc("di.sg_main.sii.1",
                                    sizeof(struct _SGEnv));
    tl_assert(env);
-   env->curr_IP       = 0;
-   env->curr_IP_known = False;
-   env->firstRef      = True;
+   env->curr_IP          = 0;
+   env->curr_IP_known    = False;
+   env->firstRef         = True;
+   env->newIRTemp_cb     = newIRTemp_cb;
+   env->newIRTemp_opaque = newIRTemp_opaque;
    return env;
 }
 
@@ -2164,7 +2173,7 @@ void sg_instrument_IRStmt ( /*MOD*/struct _SGEnv * env,
          tl_assert(env->curr_IP_known);
          if (env->firstRef) {
             instrument_mem_access( 
-               sbOut, 
+               env, sbOut, 
                st->Ist.Store.addr, 
                sizeofIRType(typeOfIRExpr(sbOut->tyenv, st->Ist.Store.data)),
                True/*isStore*/,
@@ -2181,7 +2190,7 @@ void sg_instrument_IRStmt ( /*MOD*/struct _SGEnv * env,
             tl_assert(env->curr_IP_known);
             if (env->firstRef) {
                instrument_mem_access(
-                  sbOut,
+                  env, sbOut,
                   data->Iex.Load.addr,
                   sizeofIRType(data->Iex.Load.ty),
                   False/*!isStore*/,
@@ -2207,13 +2216,13 @@ void sg_instrument_IRStmt ( /*MOD*/struct _SGEnv * env,
                dataSize = d->mSize;
                if (d->mFx == Ifx_Read || d->mFx == Ifx_Modify) {
                   instrument_mem_access( 
-                     sbOut, d->mAddr, dataSize, False/*!isStore*/,
+                     env, sbOut, d->mAddr, dataSize, False/*!isStore*/,
                      sizeofIRType(hWordTy), env->curr_IP, layout
                   );
                }
                if (d->mFx == Ifx_Write || d->mFx == Ifx_Modify) {
                   instrument_mem_access( 
-                     sbOut, d->mAddr, dataSize, True/*isStore*/,
+                     env, sbOut, d->mAddr, dataSize, True/*isStore*/,
                      sizeofIRType(hWordTy), env->curr_IP, layout
                   );
                }
@@ -2226,6 +2235,33 @@ void sg_instrument_IRStmt ( /*MOD*/struct _SGEnv * env,
          break;
       }
 
+      case Ist_CAS: {
+         /* We treat it as a read and a write of the location.  I
+            think that is the same behaviour as it was before IRCAS
+            was introduced, since prior to that point, the Vex front
+            ends would translate a lock-prefixed instruction into a
+            (normal) read followed by a (normal) write. */
+         if (env->firstRef) {
+            Int    dataSize;
+            IRCAS* cas = st->Ist.CAS.details;
+            tl_assert(cas->addr != NULL);
+            tl_assert(cas->dataLo != NULL);
+            dataSize = sizeofIRType(typeOfIRExpr(sbOut->tyenv, cas->dataLo));
+            if (cas->dataHi != NULL)
+               dataSize *= 2; /* since it's a doubleword-CAS */
+            instrument_mem_access(
+               env, sbOut, cas->addr, dataSize, False/*!isStore*/,
+               sizeofIRType(hWordTy), env->curr_IP, layout
+            );
+            instrument_mem_access(
+               env, sbOut, cas->addr, dataSize, True/*isStore*/,
+               sizeofIRType(hWordTy), env->curr_IP, layout
+            );
+            env->firstRef = False;
+         }
+         break;
+      }
+
       default:
          tl_assert(0);
 
@@ -2253,9 +2289,9 @@ void sg_instrument_final_jump ( /*MOD*/struct _SGEnv * env,
       IRExpr** args;
       IRDirty* di;
       sp_post_call_insn
-         = gen_Get_SP( sbOut, layout, sizeofIRType(hWordTy) );
+         = gen_Get_SP( env, sbOut, layout, sizeofIRType(hWordTy) );
       fp_post_call_insn
-         = gen_Get_FP( sbOut, layout, sizeofIRType(hWordTy) );
+         = gen_Get_FP( env, sbOut, layout, sizeofIRType(hWordTy) );
       tl_assert(env->curr_IP_known);
       frameBlocks = get_StackBlocks_for_IP( env->curr_IP );
       tl_assert(frameBlocks);
@@ -2416,34 +2452,34 @@ void sg_pre_thread_first_insn ( ThreadId tid ) {
 
 void sg_fini(Int exitcode)
 {
-   if (VG_(clo_verbosity) >= 2) {
+   if (VG_(clo_stats)) {
       VG_(message)(Vg_DebugMsg,
-         " sg_:  %'llu total accesses, of which:", stats__total_accesses);
+         " sg_:  %'llu total accesses, of which:\n", stats__total_accesses);
       VG_(message)(Vg_DebugMsg,
-         " sg_:     stack0: %'12llu classify",
+         " sg_:     stack0: %'12llu classify\n",
          stats__classify_Stack0);
       VG_(message)(Vg_DebugMsg,
-         " sg_:     stackN: %'12llu classify",
+         " sg_:     stackN: %'12llu classify\n",
          stats__classify_StackN);
       VG_(message)(Vg_DebugMsg,
-         " sg_:     global: %'12llu classify",
+         " sg_:     global: %'12llu classify\n",
          stats__classify_Global);
       VG_(message)(Vg_DebugMsg,
-         " sg_:    unknown: %'12llu classify",
+         " sg_:    unknown: %'12llu classify\n",
          stats__classify_Unknown);
       VG_(message)(Vg_DebugMsg,
-         " sg_:  %'llu Invars preened, of which %'llu changed",
+         " sg_:  %'llu Invars preened, of which %'llu changed\n",
          stats__Invars_preened, stats__Invars_changed);
       VG_(message)(Vg_DebugMsg,
-         " sg_:   t_i_b_MT: %'12llu", stats__t_i_b_empty);
+         " sg_:   t_i_b_MT: %'12llu\n", stats__t_i_b_empty);
       VG_(message)(Vg_DebugMsg, 
-         " sg_:     qcache: %'llu searches, %'llu probes, %'llu misses",
+         " sg_:     qcache: %'llu searches, %'llu probes, %'llu misses\n",
          stats__qcache_queries, stats__qcache_probes, stats__qcache_misses);
       VG_(message)(Vg_DebugMsg, 
-         " sg_:  htab-fast: %'llu hits",
+         " sg_:  htab-fast: %'llu hits\n",
          stats__htab_fast);
       VG_(message)(Vg_DebugMsg, 
-         " sg_:  htab-slow: %'llu searches, %'llu probes, %'llu resizes",
+         " sg_:  htab-slow: %'llu searches, %'llu probes, %'llu resizes\n",
          stats__htab_searches, stats__htab_probes, stats__htab_resizes);
    }
 }
