@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2008 Julian Seward
+   Copyright (C) 2000-2009 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@
 
 extern Bool VG_(isspace) ( Char c );
 extern Bool VG_(isdigit) ( Char c );
+extern Char VG_(tolower) ( Char c );
 
 /* ---------------------------------------------------------------------
    Converting strings to numbers
@@ -47,33 +48,37 @@ extern Bool VG_(isdigit) ( Char c );
 // accepts an initial "0x" or "0X" prefix, but only if it's followed by a
 // hex digit (if not, the '0' will be read and then it will stop on the
 // "x"/"X".)  If 'endptr' isn't NULL, it gets filled in with the first
-// non-digit char.  None of them test that the number fits into 64 bits.
+// non-digit char.  Returns 0 if no number could be converted, and 'endptr'
+// is set to the start of the string.  None of them test that the number
+// fits into 64 bits.
 //
-// Nb: if you're wondering why we don't just have a single VG_(strtol) which
+// Nb: if you're wondering why we don't just have a single VG_(strtoll) which
 // takes a base, it's because I wanted it to assert if it was given a bogus
 // base (the standard glibc one sets 'errno' in this case).  But
 // m_libcbase.c doesn't import any code, not even vg_assert. --njn
-extern Long  VG_(strtoll8)  ( Char* str, Char** endptr );
+// 
+// Nb: we also don't provide VG_(atoll*);  these functions are worse than
+// useless because they don't do any error checking and so accept malformed
+// numbers and non-numbers -- eg. "123xyz" gives 123, and "foo" gives 0!
+// If you really want that behaviour, you can use "VG_(strtoll10)(str, NULL)".
 extern Long  VG_(strtoll10) ( Char* str, Char** endptr );
 extern Long  VG_(strtoll16) ( Char* str, Char** endptr );
-extern Long  VG_(strtoll36) ( Char* str, Char** endptr );
 
-   // Convert a string to a double.  After leading whitespace is ignored,
-   // it accepts a non-empty sequence of decimal digits possibly containing
-   // a '.'.
+// Convert a string to a double.  After leading whitespace is ignored, a
+// '+' or '-' is allowed, and then it accepts a non-empty sequence of
+// decimal digits possibly containing a '.'.  Hexadecimal floats are not
+// accepted, nor are "fancy" floats (eg. "3.4e-5", "NAN").
 extern double VG_(strtod)  ( Char* str, Char** endptr );
 
-extern Long  VG_(atoll)   ( Char* str ); // base 10
-extern Long  VG_(atoll16) ( Char* str ); // base 16; leading 0x accepted
-extern Long  VG_(atoll36) ( Char* str ); // base 36
-
 /* ---------------------------------------------------------------------
    String functions and macros
    ------------------------------------------------------------------ */
 
-/* Use this for normal null-termination-style string comparison */
+/* Use this for normal null-termination-style string comparison. */
 #define VG_STREQ(s1,s2) ( (s1 != NULL && s2 != NULL \
                            && VG_(strcmp)((s1),(s2))==0) ? True : False )
+#define VG_STREQN(n,s1,s2) ( (s1 != NULL && s2 != NULL \
+                             && VG_(strncmp)((s1),(s2),(n))==0) ? True : False )
 
 extern SizeT VG_(strlen)         ( const Char* str );
 extern Char* VG_(strcat)         ( Char* dest, const Char* src );
@@ -82,17 +87,16 @@ extern Char* VG_(strpbrk)        ( const Char* s, const Char* accpt );
 extern Char* VG_(strcpy)         ( Char* dest, const Char* src );
 extern Char* VG_(strncpy)        ( Char* dest, const Char* src, SizeT ndest );
 extern Int   VG_(strcmp)         ( const Char* s1, const Char* s2 );
+extern Int   VG_(strcasecmp)     ( const Char* s1, const Char* s2 );
 extern Int   VG_(strncmp)        ( const Char* s1, const Char* s2, SizeT nmax );
+extern Int   VG_(strncasecmp)    ( const Char* s1, const Char* s2, SizeT nmax );
 extern Char* VG_(strstr)         ( const Char* haystack, Char* needle );
+extern Char* VG_(strcasestr)     ( const Char* haystack, Char* needle );
 extern Char* VG_(strchr)         ( const Char* s, Char c );
 extern Char* VG_(strrchr)        ( const Char* s, Char c );
-extern SizeT VG_(strspn)         ( const Char* s, const Char* accept );
+extern SizeT VG_(strspn)         ( const Char* s, const Char* accpt );
 extern SizeT VG_(strcspn)        ( const Char* s, const char* reject );
 
-/* Like strcmp() and strncmp(), but stop comparing at any whitespace. */
-extern Int   VG_(strcmp_ws)      ( const Char* s1, const Char* s2 );
-extern Int   VG_(strncmp_ws)     ( const Char* s1, const Char* s2, SizeT nmax );
-
 /* Like strncpy(), but if 'src' is longer than 'ndest' inserts a '\0' as the
    last character. */
 extern void  VG_(strncpy_safely) ( Char* dest, const Char* src, SizeT ndest );
@@ -115,6 +119,7 @@ extern Int   VG_(memcmp) ( const void* s1, const void* s2, SizeT n );
 #define VG_IS_4_ALIGNED(aaa_p)    (0 == (((Addr)(aaa_p)) & ((Addr)0x3)))
 #define VG_IS_8_ALIGNED(aaa_p)    (0 == (((Addr)(aaa_p)) & ((Addr)0x7)))
 #define VG_IS_16_ALIGNED(aaa_p)   (0 == (((Addr)(aaa_p)) & ((Addr)0xf)))
+#define VG_IS_32_ALIGNED(aaa_p)   (0 == (((Addr)(aaa_p)) & ((Addr)0x1f)))
 #define VG_IS_WORD_ALIGNED(aaa_p) (0 == (((Addr)(aaa_p)) & ((Addr)(sizeof(Addr)-1))))
 #define VG_IS_PAGE_ALIGNED(aaa_p) (0 == (((Addr)(aaa_p)) & ((Addr)(VKI_PAGE_SIZE-1))))
 
@@ -135,7 +140,7 @@ extern void VG_(ssort)( void* base, SizeT nmemb, SizeT size,
                         Int (*compar)(void*, void*) );
 
 /* Returns the base-2 logarithm of x.  Returns -1 if x is not a power
-   of two. */
+   of two.  Nb: VG_(log2)(1) == 0.  */
 extern Int VG_(log2) ( UInt x );
 
 // A pseudo-random number generator returning a random UInt.  If pSeed
