@@ -1,13 +1,13 @@
 
 /*--------------------------------------------------------------------*/
-/*--- Nulgrind: The null tool.                           nl_main.c ---*/
+/*--- Nulgrind: The minimal Valgrind tool.               nl_main.c ---*/
 /*--------------------------------------------------------------------*/
 
 /*
-   This file is part of Nulgrind, the simplest possible Valgrind tool,
-   which does nothing.
+   This file is part of Nulgrind, the minimal Valgrind tool,
+   which does no instrumentation or analysis.
 
-   Copyright (C) 2002-2008 Nicholas Nethercote
+   Copyright (C) 2002-2009 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -53,9 +53,9 @@ static void nl_pre_clo_init(void)
 {
    VG_(details_name)            ("Nulgrind");
    VG_(details_version)         (NULL);
-   VG_(details_description)     ("a binary JIT-compiler");
+   VG_(details_description)     ("the minimal Valgrind tool");
    VG_(details_copyright_author)(
-      "Copyright (C) 2002-2008, and GNU GPL'd, by Nicholas Nethercote.");
+      "Copyright (C) 2002-2009, and GNU GPL'd, by Nicholas Nethercote.");
    VG_(details_bug_reports_to)  (VG_BUGS_TO);
 
    VG_(basic_tool_funcs)        (nl_post_clo_init,
