@@ -1,17 +1,13 @@
 #include <unistd.h>
 #include "fdleak.h"
-int
-main (int argc, char **argv)
+
+int main (int argc, char **argv)
 {
    int fds[2];
 
-
-
-
-
-
    CLOSE_INHERITED_FDS;
 
-   pipe(fds);
+   DO( pipe(fds) );
+
    return 0;
 }
